Package: GenomicFeatures
Title: Query the gene models of a given organism/assembly
Description: Extract the genomic locations of genes, transcripts, exons,
	introns, and CDS, for the gene models stored in a TxDb object.
	A TxDb object is a small database that contains the gene models of
	a given organism/assembly. Bioconductor provides a small collection
	of TxDb objects in the form of ready-to-install TxDb packages for
	the most commonly studied organisms. Additionally, the user can
	easily make a TxDb object (or package) for the organism/assembly
	of their choice by using the tools from the txdbmaker package.
biocViews: Genetics, Infrastructure, Annotation, Sequencing,
        GenomeAnnotation
URL: https://bioconductor.org/packages/GenomicFeatures
BugReports: https://github.com/Bioconductor/GenomicFeatures/issues
Version: 1.58.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("M.", "Carlson", role="aut"),
	person("H.", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("P.", "Aboyoun", role="aut"),
	person("S.", "Falcon", role="aut"),
	person("M.", "Morgan", role="aut"),
	person("D.", "Sarkar", role="aut"),
	person("M.", "Lawrence", role="aut"),
	person("V.", "Obenchain", role="aut"),
	person("S.", "Arora", role="ctb"),
	person("J.", "MacDonald", role="ctb"),
	person("M.", "Ramos", role="ctb"),
	person("S.", "Saini", role="ctb"),
	person("P.", "Shannon", role="ctb"),
	person("L.", "Shepherd", role="ctb"),
	person("D.", "Tenenbaum", role="ctb"),
	person("D.", "Van Twisk", role="ctb"))
Depends: R (>= 3.5.0), BiocGenerics (>= 0.51.2), S4Vectors (>=
        0.17.29), IRanges (>= 2.37.1), GenomeInfoDb (>= 1.35.8),
        GenomicRanges (>= 1.55.2), AnnotationDbi (>= 1.41.4)
Imports: methods, utils, stats, DBI, XVector, Biostrings, rtracklayer
Suggests: txdbmaker, org.Mm.eg.db, org.Hs.eg.db, BSgenome,
        BSgenome.Hsapiens.UCSC.hg19 (>= 1.3.17),
        BSgenome.Celegans.UCSC.ce11, BSgenome.Dmelanogaster.UCSC.dm3
        (>= 1.3.17), mirbase.db, FDb.UCSC.tRNAs,
        TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Celegans.UCSC.ce11.ensGene,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene (>= 2.7.1),
        TxDb.Mmusculus.UCSC.mm10.knownGene (>= 3.4.7),
        TxDb.Hsapiens.UCSC.hg19.lincRNAsTranscripts,
        TxDb.Hsapiens.UCSC.hg38.knownGene (>= 3.4.6),
        SNPlocs.Hsapiens.dbSNP144.GRCh38, Rsamtools, pasillaBamSubset
        (>= 0.0.5), GenomicAlignments (>= 1.15.7), ensembldb,
        AnnotationFilter, RUnit, BiocStyle, knitr, markdown
VignetteBuilder: knitr
Collate: utils.R TxDb-schema.R TxDb-SELECT-helpers.R TxDb-class.R
        FeatureDb-class.R mapIdsToRanges.R id2name.R transcripts.R
        transcriptsBy.R transcriptsByOverlaps.R transcriptLengths.R
        exonicParts.R extendExonsIntoIntrons.R features.R microRNAs.R
        extractTranscriptSeqs.R extractUpstreamSeqs.R
        getPromoterSeq-methods.R select-methods.R nearest-methods.R
        transcriptLocs2refLocs.R coordinate-mapping-methods.R
        proteinToGenome.R coverageByTranscript.R makeTxDb.R
        makeTxDbFromUCSC.R makeTxDbFromBiomart.R makeTxDbFromEnsembl.R
        makeTxDbFromGRanges.R makeTxDbFromGFF.R makeFeatureDbFromUCSC.R
        makeTxDbPackage.R zzz.R
git_url: https://git.bioconductor.org/packages/GenomicFeatures
git_branch: RELEASE_3_20
git_last_commit: a6e2405
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29
NeedsCompilation: no
Packaged: 2024-10-30 00:50:52 UTC; biocbuild
Author: M. Carlson [aut],
  H. Pagès [aut, cre],
  P. Aboyoun [aut],
  S. Falcon [aut],
  M. Morgan [aut],
  D. Sarkar [aut],
  M. Lawrence [aut],
  V. Obenchain [aut],
  S. Arora [ctb],
  J. MacDonald [ctb],
  M. Ramos [ctb],
  S. Saini [ctb],
  P. Shannon [ctb],
  L. Shepherd [ctb],
  D. Tenenbaum [ctb],
  D. Van Twisk [ctb]
Maintainer: H. Pagès <hpages.on.github@gmail.com>
