Package: @PKGNAME@
Title: @PKGTITLE@
Description: @PKGDESCRIPTION@
Version: @PKGVERSION@
Author: @AUTHOR@
Maintainer: @MAINTAINER@
Depends: GenomicFeatures (>= @GFVERSION@)
Imports: AnnotationDbi
License: @LIC@
organism: @ORGANISM@
species: @SPECIES@
provider: @PROVIDER@
provider_version: @PROVIDERVERSION@
release_date: @RELEASEDATE@
resource_url: @SOURCEURL@
biocViews: AnnotationData, Genetics, TxDb, @ORGANISMBIOCVIEW@
