### =========================================================================
### makeFeatureDbFromUCSC()
### -------------------------------------------------------------------------

### Everything in this file has moved to txdbmaker!

supportedUCSCFeatureDbTracks <- function(genome)
{
    call_fun_in_txdbmaker("supportedUCSCFeatureDbTracks", genome=genome)
}

supportedUCSCFeatureDbTables <- function(...)
{
    call_fun_in_txdbmaker("supportedUCSCFeatureDbTables", ...)
}

UCSCFeatureDbTableSchema <- function(...)
{
    call_fun_in_txdbmaker("UCSCFeatureDbTableSchema", ...)
}

makeFeatureDbFromUCSC <- function(...)
{
    call_fun_in_txdbmaker("makeFeatureDbFromUCSC", ...)
}

